#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <curses.h>
#include <sys/time.h>
#include <pthread.h>
#include <string.h>
#include <stdint.h>
#include "hid.h"

#include "../include/usense_protocol.h"

#define cBuffLen 100
#define hBuffLen 8 /* Low speed device */
#define usbTimeout 200 // ms

unsigned char c2n(unsigned char c);

int main(int argc, char **argv) {
	initscr();
	timeout(-1);
	noecho();
	nodelay(stdscr, TRUE);
	scrollok(stdscr, TRUE);
    char n;
	unsigned char c,Run,cBuffIdx=0,cBuff[cBuffLen],hBuff[hBuffLen],hBuffIdx,nReq;
    int i;
 	if (rawhid_open(1, 0x1337, 0x4242, 0x0000, 0x0000)<= 0) {
		printw("USense device not found...Aborting\n");
	} else {
		printw("USense device Found !\n");
		Run = 1; cBuffIdx = 0;
		while(Run) {
			refresh();
			for (i=0; i<hBuffLen; i++) hBuff[i] = 0x00;
			n = rawhid_recv(0, hBuff, hBuffLen, usbTimeout);
			if (n < 0 || n > hBuffLen) {
				printw("\nerror reading, device went offline\n");
				Run = 0;
			} else {
				if(n>0) {
					/*printf("\nRX: %d bytes:\n",n);
					for (i=0; i<hBuffLen; i++) {
						printf("%02X ", hBuff[i]);
						if (i % 16 == 15) printf("\n");
					}
					printf("\n");
					*/
					if(n==3 && hBuff[0] == pGetTouch && (hBuff[1] == pAutoTX || hBuff[1] == pAck)) {
						printw("Inputs: ");
						for(i=0;i<8;i++)
							printw("%d",(hBuff[2] & (1<<i))!=0);
						printw("\n");
					} else if (n == 8 && hBuff[0] == pGetColor && hBuff[1] == pAck) {
						printw("Leds: R:%02X/%02X G:%02X/%02X B:%02X/%02X\n",
											 hBuff[2],hBuff[5],hBuff[3],hBuff[6],hBuff[4],hBuff[7]);
					} else if (n == 3 && hBuff[0] == pGetFade && hBuff[1] == pAck) {
						printw("Fade: %d\n",hBuff[2]);
					} else if (n == 6 && hBuff[0] == pGetFadeSteps && hBuff[1] == pAck) {
						printw("FadeSteps: R:%02X/%02X G: %02X/%02X B:%02X/%02X)\n",
											 hBuff[2],hBuff[5],hBuff[3],hBuff[5],hBuff[4],hBuff[5]);
					} else if (n == 4 && hBuff[0] == pGetFadeDelay && hBuff[1] == pAck) {
						printw("FadeDelay: %02X/%02X\n",hBuff[2],hBuff[3]);
					} else if (n == 3 && hBuff[0] == pGetTouchINT && hBuff[1] == pAck) {
						printw("TouchINT: %d\n",hBuff[2]);
					} else if (n == 3 && hBuff[0] == pGetTouchACT && hBuff[1] == pAck) {
						printw("TouchACT: %d\n",hBuff[2]);
					} else if (n == 4 && hBuff[0] == pGetFadeMode && hBuff[1] == pAck) {
						printw("FadeMode: %02X. Next: %02X\n",hBuff[2],hBuff[3]);
					}
				} 
				c=getch();
				if (c == 'q' && !cBuffIdx)
					Run=0x00;
				else {
					if(c == '\n') {
						printw("\n");
						//~ printw("User: %d bytes: ",cBuffIdx);
						//~ for(i=0;i<cBuffIdx;i++) printw("%c",cBuff[i]); printw("\n");
						/*for (i=0; i<cBuffIdx; i++) {
							printw("%02X ", cBuff[i]);
							if (i % 16 == 15) printw("\n");
						}
						*/
						c = 0; nReq = 0x00; hBuffIdx = 0x00;
						for(i=0;i<hBuffLen;i++) hBuff[i] = 0;
						/*for(i=0;i<hBuffLen && i<cBuffIdx;i++)	hBuff[i] = cBuff[i];*/
						if(cBuffIdx) {
							nReq = 1;
							if(cBuff[0] == 'z') {							/* Get Fade ***********/
								hBuff[0] = pGetFade;
							} else if(cBuff[0] == 's' && cBuffIdx >=7) {	/* Fade On ************/
								hBuff[0] = pFadeOn;
								hBuff[1] = (c2n(cBuff[1])<<4)|c2n(cBuff[2]);
								hBuff[2] = (c2n(cBuff[3])<<4)|c2n(cBuff[4]);
								hBuff[3] = (c2n(cBuff[5])<<4)|c2n(cBuff[6]);
								printw("Fade on (max brightness: %02X%02X%02X)\n",
																		hBuff[1],hBuff[2],hBuff[3]);
							} else if(cBuff[0] == 'x') {					/* Fade Off ***********/
								hBuff[0] = pFadeOff;
								printw("Fade off\n");
								
							} else if(cBuff[0] == 'e') {					/* Get LEDs ***********/
								hBuff[0] = pGetColor;
							} else if(cBuff[0] == 'd' && cBuffIdx >= 7) { 	/* Fixed color ********/
								hBuff[0] = pSetColor;
								hBuff[1] = (c2n(cBuff[1])<<4)|c2n(cBuff[2]);
								hBuff[2] = (c2n(cBuff[3])<<4)|c2n(cBuff[4]);
								hBuff[3] = (c2n(cBuff[5])<<4)|c2n(cBuff[6]);
								printw("Leds set to R:%02X G:%02X B:%02X\n",
																		hBuff[1],hBuff[2],hBuff[3]);
							} else if(cBuff[0] == 'c') {					/* LEDs Off ***********/
								hBuff[0] = pClear;
								printw("Leds off\n");
								
							} else if(cBuff[0] == 'f' && cBuffIdx >= 7) { 	/* Set Fade Steps *****/
								hBuff[0] = pSetFadeSteps;
								hBuff[1] = (c2n(cBuff[1])<<4)|c2n(cBuff[2]);
								hBuff[2] = (c2n(cBuff[3])<<4)|c2n(cBuff[4]);
								hBuff[3] = (c2n(cBuff[5])<<4)|c2n(cBuff[6]);
								printw("Fade steps set to R:%02X G:%02X B:%02X\n",
																		hBuff[1],hBuff[2],hBuff[3]);
							} else if(cBuff[0] == 'r') {					/* Get Fade Steps *****/
								hBuff[0] = pGetFadeSteps;
							
							} else if(cBuff[0] == 'k' && cBuffIdx >= 3) { 	/* Set Fade Delay *****/
								hBuff[0] = pSetFadeDelay;
								hBuff[1] = (c2n(cBuff[1])<<4)|c2n(cBuff[2]);
								printw("Fade delay set to %02X\n", hBuff[1]);
							} else if(cBuff[0] == 'i') {					/* Get Fade Delay *****/
								hBuff[0] = pGetFadeDelay;
								
							} else if(cBuff[0] == 'v') {					/* Get Touch **********/
								hBuff[0] = pGetTouch;
								
							} else if(cBuff[0] == 't') {					/* Get Touch INT ******/
								hBuff[0] = pGetTouchINT;
							} else if(cBuff[0] == 'g') {					/* Enable Touch INT ***/
								hBuff[0] = pTouchINTon;
								printw("Enable Touch INT\n");
							} else if(cBuff[0] == 'b') {					/* Disable Touch INT **/
								hBuff[0] = pTouchINToff;
								printw("Disable Touch INT\n");
								
							} else if(cBuff[0] == 'y') {					/* Get Touch ACT ******/
								hBuff[0] = pGetTouchACT;
							} else if(cBuff[0] == 'h') {					/* Enable Touch ACT ***/
								hBuff[0] = pTouchACTon;
								printw("Enable Touch ACT\n");
							} else if(cBuff[0] == 'n') {					/* Disable Touch ACT **/
								hBuff[0] = pTouchACToff;
								printw("Disable Touch ACT\n");

							} else if(cBuff[0] == 'u') {					/* Get Fade Mode ******/
								hBuff[0] = pGetFadeMode;
							} else if(cBuff[0] == 'j') {					/* Set Fade Mode ******/
								hBuff[0] = pSetFadeMode;
								hBuff[1] = (c2n(cBuff[1])<<4)|c2n(cBuff[2]);
								printw("Set FadeMode to %X\n",hBuff[1]);

							} else if(cBuff[0] == 'p') {					/* Reset **************/
								hBuff[0] = pReset;
								printw("Reset triggered\n");
							} else {										/* Unknown stuff ******/
								nReq = 0;
								printw("Unknown command...\n");
							}
						}
						if(nReq) {
							//~ printw("\nTX: %d bytes:\n",hBuffLen);
							//~ for (i=0; i<cBuffIdx; i++) hBuff[i] = cBuff[i];
							//~ for (i=cBuffIdx; i<hBuffLen; i++) hBuff[i] = 0;
							/*for (i=0; i<hBuffLen; i++) {
								printw("%02X ", hBuff[i]);
								if (i % 16 == 15) printw("\n");
							}*/
							//~ printw("\n");
							rawhid_send(0, hBuff, hBuffLen, usbTimeout);
						}
						cBuffIdx = 0x00;
					} else {
						if(c >= '0' && c <= 'z') {
							printw("%c",c);
							if(cBuffIdx < cBuffLen) {
								cBuff[cBuffIdx++] = c;
							}
							
						}
					}
				}
			}
		}
		printw("\nBye!\n");
		rawhid_close(0);
	}
	endwin();
    return EXIT_SUCCESS;
}

unsigned char c2n(unsigned char c) {
	if(c >= '0' && c <= '9')
		c -= '0';
	else if(c >= 'A' && c <= 'F')
		c -= 'A'-10;
	else if(c >= 'a' && c <= 'f')
		c -= 'a'-10;
	return c;
}
