#include <avr/io.h>
#include <avr/wdt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>

#include "usbdrv/usbdrv.h"
#include "ws2812.h"
#include "touch.h"
#include "usense_protocol.h"

#include "main.h"

static uint8_t usbIN[usbLen],usbOUT[usbLen], usbINwIdx,usbOUTwIdx;				   /* USB Buffers */
static cRGB Leds[1];															   /* LEDs values */
static sRGB sLeds[1];															   /* LEDs speeds */

static const uint8_t Seq[] = {tLeft,tRight,tLeft,tBoth};					   /* Secret Sequence */

int main(void) {
	uint8_t i,j,Mode,FadeMode,nFadeMode;
	uint8_t lTps = 0,lDelay,lCDelay;													  /* LEDs */
	int8_t inc;
	cRGB lBmax,lSteps;
	uint16_t sample,sRefs[nInputs];														 /* Touch */
	uint8_t tInput=0, tPrevInput = 0, tDeb=0, tSeqIdx=0;
	
	
		/* Init CPU *******************************************************************************/
    wdt_enable(WDTO_1S);
	PORTA = 0;						
	DDRA |= 0b111100;
	
		/* Init LEDs ******************************************************************************/
	for(i=0;i<nLEDs;i++) {
		Leds[i].r = Leds[i].g = Leds[i].b = 0;
		sLeds[i].r = 0; sLeds[i].g = sLeds[i].b = 0;
	}
		/* Init Touch *****************************************************************************/
    touch_init();
	for(i=0;i<nInputs;i++)
		sRefs[i] = touch_measure(&Btns[i]);
		
		/* Init USB *******************************************************************************/
	usbInit();
	usbDeviceDisconnect();
	_delay_ms(300);
    usbDeviceConnect();
	
		/* Time to party ! ************************************************************************/	
	Mode = mFade | mTouchAct;
	FadeMode = pDimAllMode;
	nFadeMode = pRGBCycleMode;
	lBmax.r = lBmax.g = lBmax.b = LBmax;
	lSteps.r = lSteps.g = lSteps.b = 1;
	lDelay = 0x00;
	
	sei();
	while (1) {
		wdt_reset();
			/* USB management *********************************************************************/
		usbPoll();
					/* Received data ? ************************************************************/
		if(usbINwIdx) {
			/* for(usbOUTwIdx=0;usbOUTwIdx<usbINwIdx;usbOUTwIdx++) 
				usbOUT[usbOUTwIdx] = usbIN[usbOUTwIdx]; */
			usbOUTwIdx = 0;
			usbOUT[usbOUTwIdx++] = usbIN[0];
			usbOUT[usbOUTwIdx++] = pAck;										  /* Default: Ack */
			
			if(usbIN[0] == pFadeOn) {					/* Fade On ********************************/
				for(i=0;i<nLEDs;i++) {
					Leds[i].r = Leds[i].g = Leds[i].b = 0;
					sLeds[i].r = 1; sLeds[i].g = sLeds[i].b = 0;
				}
				lBmax.r = usbIN[1];
				lBmax.g = usbIN[2];
				lBmax.b = usbIN[3];
				Mode |= mFade;
			} else if(usbIN[0] == pGetFade) {			/* Get Fade *******************************/
				usbOUT[usbOUTwIdx++] = ((Mode & mFade) == mFade);
			} else if(usbIN[0] == pSetFadeSteps) {		/* Fade Steps *****************************/
				lSteps.r = usbIN[1] < LStepsmax ? usbIN[1] : LStepsmax;
				lSteps.g = usbIN[2] < LStepsmax ? usbIN[2] : LStepsmax;
				lSteps.b = usbIN[3] < LStepsmax ? usbIN[3] : LStepsmax;
			} else if(usbIN[0] == pGetFadeSteps) {		/* Get Fade Steps *************************/
				usbOUT[usbOUTwIdx++] = lSteps.r;
				usbOUT[usbOUTwIdx++] = lSteps.g;
				usbOUT[usbOUTwIdx++] = lSteps.b;
				usbOUT[usbOUTwIdx++] = LStepsmax;
			} else if(usbIN[0] == pFadeOff) {			/* Fade Off *******************************/
				Mode &= ~mFade;
			} else if(usbIN[0] == pGetFadeDelay) {		/* Get Fade Delay *************************/
				usbOUT[usbOUTwIdx++] = lDelay;
				usbOUT[usbOUTwIdx++] = LDelaymax;
			} else if(usbIN[0] == pSetFadeDelay) {		/* Set Fade Delay *************************/
				lDelay = usbIN[1] < LDelaymax ? usbIN[1] : LDelaymax;

			} else if(usbIN[0] == pSetColor) {			/* Fixed color ****************************/
				for(i=0;i<nLEDs;i++) {
					Leds[i].r = usbIN[1];
					Leds[i].g = usbIN[2];
					Leds[i].b = usbIN[3];
					sLeds[i].r = sLeds[i].g = sLeds[i].b = 0;
				}
				Mode &= ~mFade;
			} else if(usbIN[0] == pGetColor) { 			/* Get LEDs Color *************************/
				for(i=0;i<nLEDs && usbOUTwIdx < usbLen;i++) {
					usbOUT[usbOUTwIdx++] = Leds[i].r;
					usbOUT[usbOUTwIdx++] = Leds[i].g;
					usbOUT[usbOUTwIdx++] = Leds[i].b;
					usbOUT[usbOUTwIdx++] = lBmax.r;
					usbOUT[usbOUTwIdx++] = lBmax.g;
					usbOUT[usbOUTwIdx++] = lBmax.b;
				}
			} else if(usbIN[0] == pClear) {				/* LEDs Off *******************************/
				for(i=0;i<nLEDs;i++) {
					Leds[i].r = Leds[i].g = Leds[i].b = 0;
					sLeds[i].r = sLeds[i].g = sLeds[i].b = 0;
				}
				Mode &= ~mFade;

			} else if(usbIN[0] == pGetTouch) {			/* Get Touch value ************************/
				usbOUT[usbOUTwIdx++] = tInput;
			} else if(usbIN[0] == pTouchINTon) {		/* Enable INT *****************************/
					Mode |= mTouchPub;
			} else if(usbIN[0] == pTouchINToff) {		/* Disable INT ****************************/
					Mode &= ~mTouchPub;
			} else if(usbIN[0] == pGetTouchINT) {		/* Get TouchINT ***************************/
				usbOUT[usbOUTwIdx++] = ((Mode & mTouchPub) == mTouchPub);
			} else if(usbIN[0] == pTouchACTon) {		/* Enable ACT *****************************/
					Mode |= mTouchAct;
			} else if(usbIN[0] == pTouchACToff) {		/* Disable ACT ****************************/
					Mode &= ~mTouchAct;
			} else if(usbIN[0] == pGetTouchACT) {		/* Get TouchACT ***************************/
					usbOUT[usbOUTwIdx++] = ((Mode & mTouchAct) == mTouchAct);
			
			} else if(usbIN[0] == pGetFadeMode) {		/* Get Fade Mode **************************/
				usbOUT[usbOUTwIdx++] = FadeMode;
				usbOUT[usbOUTwIdx++] = nFadeMode;
			} else if(usbIN[0] == pSetFadeMode) {		/* Set Fade Mode **************************/				
				if(usbIN[1] == pRGBCycleMode || usbIN[1] == pEachFadeMode) {
					FadeMode = pDimAllMode;
					nFadeMode = usbIN[1];
				}

			} else if(usbIN[0] == pReset) {				/* Reset (force Watchdog timeout) *********/
				while(1);
			} else {
				usbOUT[1] = 0xFF;														  /* NACK */
			}
			usbINwIdx=0;									  /* Data Processed, flush the buffer */
		}
					/* Send data ? ****************************************************************/
		if(usbOUTwIdx) {
			usbSetInterrupt(usbOUT,usbOUTwIdx);
			usbOUTwIdx=0;
		}
			/* LEDs management ********************************************************************/
		if(Mode & mFade && lCDelay++ >= lDelay) {
			lCDelay = 0;
			j=1;
			for(i=0;i<nLEDs;i++) {
				/* Colors Update & Limiting ***************************************************/
																						   /* Red */
				inc = sLeds[i].r*lSteps.r;
				if(sLeds[i].r > 0 && lBmax.r && Leds[i].r > lBmax.r-inc)	{			   /* Top */
					Leds[i].r = lBmax.r;
				} else if(sLeds[i].r < 0 && Leds[i].r < -inc) { 					   	/* Bottom */
					Leds[i].r = 0;
				} else
					Leds[i].r += inc;
																						 /* Green */
				inc = sLeds[i].g*lSteps.g;
				if(sLeds[i].g > 0 && lBmax.g && Leds[i].g > lBmax.g-inc) {				   /* Top */
					Leds[i].g = lBmax.g;
				} else if(sLeds[i].g < 0 && Leds[i].g < -inc) { 					   	/* Bottom */
					Leds[i].g = 0;
				} else
					Leds[i].g += inc;
																						  /* Blue */
				inc = sLeds[i].b*lSteps.b;
				if(sLeds[i].b > 0 && lBmax.b && Leds[i].b > lBmax.b-inc) {				   /* Top */
					Leds[i].b = lBmax.b;
				} else if(sLeds[i].b < 0 && Leds[i].b < -inc) { 					   	/* Bottom */
					Leds[i].b = 0;
				} else
					Leds[i].b += inc;
				
				if(FadeMode == pRGBCycleMode) {
					/* Color Cycling **************************************************************/
					if(sLeds[i].r > 0 && Leds[i].r >= lBmax.r) {			   /* Red reaches TOP */
						sLeds[i].r = -1;										 /* Red goes DOWN */
						sLeds[i].g = 1;											/*  Green goes UP */
					} 
					if(sLeds[i].g > 0 && Leds[i].g >= lBmax.g) {			 /* Green Reaches TOP */
						sLeds[i].g = -1;									   /* Green goes DOWN */
						sLeds[i].b = 1;											  /* Blue goes UP */
					} 
					if(sLeds[i].b > 0 && Leds[i].b >= lBmax.b) {			  /* Blue reaches TOP */
						sLeds[i].b = -1;									    /* Blue goes DOWN */
						sLeds[i].r = 1;											   /* Red goes UP */
					}
					j=0;
				} else if(FadeMode == pEachFadeMode) {
					/* Color Cycling **************************************************************/
					if(Leds[i].r >= lBmax.r && sLeds[i].r > 0) {			   /* Red reaches TOP */
						sLeds[i].r = -1;										 /* Red goes DOWN */
					} else if(Leds[i].r == 0 && sLeds[i].r < 0) {			/* Red reaches BOTTOM */
						sLeds[i].r = 0;											 /* Red goes STDY */
						sLeds[i].g = 1;											/*  Green goes UP */
					} else if(Leds[i].g >= lBmax.g && sLeds[i].g > 0) {		 /* Green Reaches TOP */
						sLeds[i].g = -1;									   /* Green goes DOWN */
					} else if(Leds[i].g == 0 && sLeds[i].g < 0) {		  /* Green reaches BOTTOM */
						sLeds[i].g = 0;										   /* Green goes STDY */
						sLeds[i].b = 1;											  /* Blue goes UP */
					} else if(Leds[i].b >= lBmax.b && sLeds[i].b > 0) {		  /* Blue reaches TOP */
						sLeds[i].b = -1;									    /* Blue goes DOWN */
					} else if(Leds[i].b == 0 && sLeds[i].b < 0) {		   /* Blue reaches BOTTOM */
						sLeds[i].b = 0;										    /* Blue goes STDY */
						sLeds[i].r = 1;											   /* Red goes UP */
					}
					j=0;
				} else if(FadeMode == pDimAllMode) {
					if(Leds[i].r > 0) {											  /* Set Red DOWN */
						j=0;
						sLeds[i].r = -1;
					} else if(Leds[i].r == 0)									  /* Set Red STDY */
						sLeds[i].r = 0;
					if(Leds[i].g > 0) {											/* Set Green DOWN */
						j=0;
						sLeds[i].g = -1;
					} else if(Leds[i].g == 0)									/* Set Green STDY */
						sLeds[i].g = 0;
					if(Leds[i].b > 0) {											 /* Set Blue DOWN */
						j=0;
						sLeds[i].b = -1;
					} else if(Leds[i].b == 0)									 /* Set Blue STDY */
						sLeds[i].b = 0;
				}
			}
			if(j && nFadeMode) {						/* NextMode selected and ready to proceed */
				FadeMode = nFadeMode;
				nFadeMode = 0;
				if(FadeMode == pRGBCycleMode || FadeMode == pEachFadeMode)	/* Prepare the Speeds */
					for(i=0;i<nLEDs;i++)
						sLeds[i].r = 1;
			}
		}
		ws2812_setleds(Leds,nLEDs);											 /* Spread the news ! */

			/* Touch Input ************************************************************************/
		tInput = 0;
		for(i=0;i<nInputs;i++) {
			sample = touch_measure(&Btns[i]);						   /* Sample the Touch sensor */
			if(sample > sRefs[i] + TFuzz) {											 /* Pressed ? */
				tInput |= 1<<i;												/* Add to inputs list */
			}
		}
		
		if(lTps) {																 /* LED timeout ? */
			if(!--lTps) PORTA &= ~_BV(3);									   /* Turn of the LED */
		} else {
										/*  USB publish *******************************************/
			if(tInput != tPrevInput && (Mode & mTouchPub)) {
				tPrevInput = tInput;
				usbOUT[0] = pGetTouch; usbOUT[1] = pAutoTX;	usbOUT[2] = tInput;	/* Prepare Buffer */
				usbOUTwIdx = 3;
			}
			if(tInput == tBoth && tDeb++>TDeb) {						  /* Both pads, debounced */
				if(!nFadeMode) {												   /* Switch mode */
					if(FadeMode == pRGBCycleMode) {
						nFadeMode = pEachFadeMode;
					} else if(FadeMode == pEachFadeMode) {
						nFadeMode = pRGBCycleMode;
					}
					FadeMode = pDimAllMode;
				}
				tDeb = 0; PORTA |= _BV(3); lTps = LTps;
			} else if(tInput == tLeft && tDeb++>TDeb) {					   /* Left pad, debounced */
				if(Mode & mTouchAct) {
					if(lSteps.r < LStepsmax)	lSteps.r++;
					if(lSteps.g < LStepsmax)	lSteps.g++;
					if(lSteps.b < LStepsmax)	lSteps.b++;
				}
				tDeb = 0;	PORTA |= _BV(3); lTps = LTps;
			} else if(tInput == tRight && tDeb++>TDeb) {		  		  /* Right pad, debounced */
				if(Mode & mTouchAct) {
					if(lSteps.r)	lSteps.r--;
					if(lSteps.g)	lSteps.g--;
					if(lSteps.b)	lSteps.b--;
				}
				tDeb = 0;	PORTA |= _BV(3); lTps = LTps;
			}
			if(lTps) {
				tSeqIdx = tInput == Seq[tSeqIdx] ? tSeqIdx+1 : 0;
				if(tSeqIdx == nSeq) {
					tSeqIdx=0;
					_delay_ms(500); wdt_reset(); PORTA &= ~_BV(3); 
					_delay_ms(500); wdt_reset(); PORTA |= _BV(3);
					_delay_ms(500); wdt_reset(); PORTA &= ~_BV(3); 
					_delay_ms(500); wdt_reset(); PORTA |= _BV(3);
					_delay_ms(500); wdt_reset(); PORTA &= ~_BV(3); 
					_delay_ms(500); wdt_reset(); PORTA |= _BV(3);
					_delay_ms(500); wdt_reset(); PORTA &= ~_BV(3); 
					_delay_ms(500); wdt_reset(); PORTA |= _BV(3);
				}
			}
		}
	}
}

	/* USB Setup **********************************************************************************/
#define USB_TX_SIZE 8
#define USB_RX_SIZE	8

const PROGMEM char usbDescriptorConfiguration[USB_CFG_DESCR_PROPS_CONFIGURATION] = {
	 // configuration descriptor, USB spec 9.6.3, page 264-266, Table 9-10
    9,                                      // bLength
    2,                                      // bDescriptorType
    9 + 9 + 9 + 7 + 7,                      // wTotalLength LSB
	0,				 						// wTotalLength MSB
    1,                          			// bNumInterfaces
    1,                                      // bConfigurationValue
    0,                                      // iConfiguration
    0xC0,                             		// bmAttributes
    50,  						            // bMaxPower
	// Interface descriptor, USB spec 9.6.5, page 267-269, Table 9-12
    9,                                      // bLength
    4,                     					// bDescriptorType
    0,                       				// bInterfaceNumber
    0,                                      // bAlternateSetting
    2,                                      // bNumEndpoints
    0x03,              						// bInterfaceClass (0x03 = HID)
    0x00, 						            // bInterfaceSubClass
    0x00,					         	    // bInterfaceProtocol
    0,                                      // iInterface
	// HID interface descriptor, HID 1.11 spec, section 6.2.1
    9,                                      // bLength
    0x21,                 			        // bDescriptorType
    0x01, 0x01,                             // bcdHID
    0,                                      // bCountryCode
    1,                                      // bNumDescriptors
    0x22,                                   // bDescriptorType
    USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH,   // wDescriptorLength LSB
	0,										// wDescriptorLength MSB
	// endpoint descriptor, USB spec 9.6.6, page 269-271, Table 9-13
    7,                                      // bLength
    5,   				                    // bDescriptorType
    0x81,              						// bEndpointAddress(IN: 0x80 | addr)
    0x03,                                   // bmAttributes (0x03=intr)
    USB_TX_SIZE,	                        // wMaxPacketSize LSB
	0,                      				// wMaxPacketSize MSB
    USB_CFG_INTR_POLL_INTERVAL,             // bInterval
	// endpoint descriptor, USB spec 9.6.6, page 269-271, Table 9-13
	7,                                      // bLength
    USBDESCR_ENDPOINT,                      // bDescriptorType
    0x02,              						// bEndpointAddress (OUT: addr)
    0x03,                                   // bmAttributes (0x03=intr)
    USB_RX_SIZE,	                        // wMaxPacketSize LSB
	0,                      				// wMaxPacketSize MSB
    USB_CFG_INTR_POLL_INTERVAL,             // bInterval
};

const PROGMEM char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = {
    0x06, 0xab, 0xff,              			// UsagePage
	0x0A, 0x00, 0x02,			   			// Usage
    0xa1, 0x01,                    			// Collection	
		0x75, 0x08,                    		// Report size 8bits
		0x15, 0x00,                    		// Logical minimum 0
		0x26, 0xff, 0x00,              		// Logical maximum 255
											// EP IN
		0x95, USB_RX_SIZE,             		// Report count
		0x09, 0x01,                    		// Usage
		0x81, 0x02,							// Input (array)
											// EP OUT
		0x95, USB_RX_SIZE,             		// Report count
		0x09, 0x02,                    		// Usage
		0x91, 0x02,                    		// Output (array)
	
		0x95, 0x04,                    		// Report count (global)
        0x09, 0x76,                    		// Usage (local)
		0xb1, 0x02,							// Feature
    0xc0                           			// End Collection
};

usbMsgLen_t usbFunctionSetup(uint8_t data[8]) {
    //~ usbRequest_t *rq = (void *)data;
    return 0;
}

void usbFunctionWriteOut(uchar *data, uchar len) {
	for(usbINwIdx = 0; usbINwIdx < len && usbINwIdx < usbLen; usbINwIdx++)
		usbIN[usbINwIdx] = *(data+usbINwIdx);
}
